@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -20,32 +20,45 @@ from .constants import *
 
 # pylint: enable=wildcard-import
 
+from .agent import SSHAgentClient, connect_agent
+
 from .auth_keys import SSHAuthorizedKeys
 from .auth_keys import import_authorized_keys, read_authorized_keys
 
-from .channel import SSHClientChannel, SSHServerChannel, SSHTCPChannel
+from .channel import SSHClientChannel, SSHServerChannel
+from .channel import SSHTCPChannel, SSHUNIXChannel
+
+from .client import SSHClient
 
-from .connection import SSHClient, SSHServer
 from .connection import SSHClientConnection, SSHServerConnection
 from .connection import create_connection, create_server, connect, listen
 
+from .known_hosts import SSHKnownHosts
+from .known_hosts import import_known_hosts, read_known_hosts
+from .known_hosts import match_known_hosts
+
 from .listener import SSHListener
 
 from .logging import logger
 
 from .misc import Error, DisconnectError, ChannelOpenError
+from .misc import PasswordChangeRequired
 from .misc import BreakReceived, SignalReceived, TerminalSizeChanged
 
 from .pbe import KeyEncryptionError
 
-from .public_key import SSHKey, SSHCertificate, KeyImportError, KeyExportError
+from .public_key import SSHKey, SSHKeyPair, SSHCertificate
+from .public_key import KeyImportError, KeyExportError
 from .public_key import import_private_key, import_public_key
 from .public_key import import_certificate
 from .public_key import read_private_key, read_public_key, read_certificate
 from .public_key import read_private_key_list, read_public_key_list
 from .public_key import read_certificate_list
 
-from .session import SSHClientSession, SSHServerSession, SSHTCPSession
+from .session import SSHClientSession, SSHServerSession
+from .session import SSHTCPSession, SSHUNIXSession
+
+from .server import SSHServer
 
 from .sftp import SFTPClient, SFTPServer, SFTPFile, SFTPError
 from .sftp import SFTPAttrs, SFTPVFSAttrs, SFTPName
