@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -15,7 +15,7 @@
 import asyncio
 
 from .constants import DISC_PROTOCOL_ERROR
-from .misc import DisconnectError
+from .misc import DisconnectError, PasswordChangeRequired
 from .packet import Boolean, Byte, String, UInt32, SSHPacketHandler
 from .saslprep import saslprep, SASLPrepError
 
@@ -42,9 +42,18 @@ _server_auth_handlers = {}
 class _Auth(SSHPacketHandler):
     """Parent class for authentication"""
 
-    def __init__(self):
+    def __init__(self, conn, coro):
+        self._conn = conn
         self._coro = None
 
+        self.create_task(coro)
+
+    def create_task(self, coro):
+        """Create an asynchronous auth task"""
+
+        self.cancel()
+        self._coro = self._conn.create_task(coro)
+
     def cancel(self):
         """Cancel any authentication in progress"""
 
@@ -52,15 +61,14 @@ class _Auth(SSHPacketHandler):
             self._coro.cancel()
             self._coro = None
 
+
 class _ClientAuth(_Auth):
     """Parent class for client authentication"""
 
     def __init__(self, conn, method):
-        super().__init__()
-
-        self._conn = conn
         self._method = method
-        self._coro = asyncio.async(self._start())
+
+        super().__init__(conn, self._start())
 
     @asyncio.coroutine
     def _start(self):
@@ -75,10 +83,12 @@ class _ClientAuth(_Auth):
     def auth_failed(self):
         """Callback when auth fails"""
 
+    @asyncio.coroutine
     def send_request(self, *args, key=None):
         """Send a user authentication request"""
 
-        self._conn.send_userauth_request(self._method, *args, key=key)
+        yield from self._conn.send_userauth_request(self._method,
+                                                    *args, key=key)
 
 
 class _ClientNullAuth(_ClientAuth):
@@ -88,9 +98,7 @@ class _ClientNullAuth(_ClientAuth):
     def _start(self):
         """Start client null authentication"""
 
-        self.send_request()
-
-    packet_handlers = {}
+        yield from self.send_request()
 
 
 class _ClientPublicKeyAuth(_ClientAuth):
@@ -100,15 +108,24 @@ class _ClientPublicKeyAuth(_ClientAuth):
     def _start(self):
         """Start client public key authentication"""
 
-        self._alg, self._key, self._key_data = \
-            yield from self._conn.public_key_auth_requested()
+        self._keypair = yield from self._conn.public_key_auth_requested()
 
-        if self._alg is None:
+        if self._keypair is None:
             self._conn.try_next_auth()
             return
 
-        self.send_request(Boolean(False), String(self._alg),
-                          String(self._key_data))
+        yield from self.send_request(Boolean(False),
+                                     String(self._keypair.algorithm),
+                                     String(self._keypair.public_data))
+
+    @asyncio.coroutine
+    def _send_signed_request(self):
+        """Send signed public key request"""
+
+        yield from self.send_request(Boolean(True),
+                                     String(self._keypair.algorithm),
+                                     String(self._keypair.public_data),
+                                     key=self._keypair)
 
     def _process_public_key_ok(self, pkttype, packet):
         """Process a public key ok response"""
@@ -119,11 +136,11 @@ class _ClientPublicKeyAuth(_ClientAuth):
         key_data = packet.get_string()
         packet.check_end()
 
-        if algorithm != self._alg or key_data != self._key_data:
+        if (algorithm != self._keypair.algorithm or
+                key_data != self._keypair.public_data):
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Key mismatch')
 
-        self.send_request(Boolean(True), String(algorithm),
-                          String(key_data), key=self._key)
+        self.create_task(self._send_signed_request())
         return True
 
     packet_handlers = {
@@ -144,7 +161,7 @@ class _ClientKbdIntAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
-        self.send_request(String(''), String(submethods))
+        yield from self.send_request(String(''), String(submethods))
 
     @asyncio.coroutine
     def _receive_challenge(self, name, instruction, lang, prompts):
@@ -193,9 +210,8 @@ class _ClientKbdIntAuth(_ClientAuth):
 
             prompts.append((prompt, echo))
 
-        self.cancel()
-        self._coro = asyncio.async(self._receive_challenge(name, instruction,
-                                                           lang, prompts))
+        self.create_task(self._receive_challenge(name, instruction,
+                                                 lang, prompts))
 
         return True
 
@@ -222,13 +238,13 @@ class _ClientPasswordAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
-        self.send_request(Boolean(False), String(password))
+        yield from self.send_request(Boolean(False), String(password))
 
     @asyncio.coroutine
-    def _change_password(self):
+    def _change_password(self, prompt, lang):
         """Start password change"""
 
-        result = yield from self._conn.password_change_requested()
+        result = yield from self._conn.password_change_requested(prompt, lang)
 
         if result == NotImplemented:
             # Password change not supported - move on to the next auth method
@@ -239,9 +255,9 @@ class _ClientPasswordAuth(_ClientAuth):
 
         self._password_change = True
 
-        self.send_request(Boolean(True),
-                          String(old_password.encode('utf-8')),
-                          String(new_password.encode('utf-8')))
+        yield from self.send_request(Boolean(True),
+                                     String(old_password.encode('utf-8')),
+                                     String(new_password.encode('utf-8')))
 
     def auth_succeeded(self):
         if self._password_change:
@@ -268,8 +284,8 @@ class _ClientPasswordAuth(_ClientAuth):
             raise DisconnectError(DISC_PROTOCOL_ERROR,
                                   'Invalid password change request') from None
 
-        self.cancel()
-        self._coro = asyncio.async(self._change_password())
+        self.auth_failed()
+        self.create_task(self._change_password(prompt, lang))
 
         return True
 
@@ -282,11 +298,9 @@ class _ServerAuth(_Auth):
     """Parent class for server authentication"""
 
     def __init__(self, conn, username, packet):
-        super().__init__()
-
-        self._conn = conn
         self._username = username
-        self._coro = asyncio.async(self._start(packet))
+
+        super().__init__(conn, self._start(packet))
 
     @asyncio.coroutine
     def _start(self, packet):
@@ -437,8 +451,7 @@ class _ServerKbdIntAuth(_ServerAuth):
 
         packet.check_end()
 
-        self.cancel()
-        self._coro = asyncio.async(self._validate_response(responses))
+        self.create_task(self._validate_response(responses))
 
     packet_handlers = {
         MSG_USERAUTH_INFO_RESPONSE: _process_info_response
@@ -470,12 +483,23 @@ class _ServerPasswordAuth(_ServerAuth):
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid password auth '
                                   'request') from None
 
-        # TODO: Handle password change request
+        try:
+            if password_change:
+                result = yield from self._conn.change_password(self._username,
+                                                               password,
+                                                               new_password)
+            else:
+                result = \
+                    yield from self._conn.validate_password(self._username,
+                                                            password)
 
-        if (yield from self._conn.validate_password(self._username, password)):
-            self.send_success()
-        else:
-            self.send_failure()
+            if result:
+                self.send_success()
+            else:
+                self.send_failure()
+        except PasswordChangeRequired as exc:
+            self._conn.send_packet(Byte(MSG_USERAUTH_PASSWD_CHANGEREQ),
+                                   String(exc.prompt), String(exc.lang))
 
 
 def register_auth_method(alg, client_handler, server_handler):
