@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -21,12 +21,8 @@ from .constants import MSG_CHANNEL_EOF, MSG_CHANNEL_CLOSE, MSG_CHANNEL_REQUEST
 from .constants import MSG_CHANNEL_SUCCESS, MSG_CHANNEL_FAILURE
 from .constants import OPEN_CONNECT_FAILED, PTY_OP_RESERVED, PTY_OP_END
 from .constants import OPEN_REQUEST_PTY_FAILED, OPEN_REQUEST_SESSION_FAILED
-from .misc import ChannelOpenError, DisconnectError
+from .misc import ChannelOpenError, DisconnectError, map_handler_name
 from .packet import Boolean, Byte, String, UInt32, SSHPacketHandler
-from .sftp import SFTPServerSession
-
-
-_EOF = object()
 
 
 class SSHChannel(SSHPacketHandler):
@@ -73,12 +69,18 @@ class SSHChannel(SSHPacketHandler):
 
         self._open_waiter = None
         self._request_waiters = []
-        self._close_waiters = []
+
+        self._close_event = asyncio.Event()
 
         self.set_write_buffer_limits()
 
         self._recv_chan = conn.add_channel(self)
 
+    def get_connection(self):
+        """Return the connection used by this channel"""
+
+        return self._conn
+
     def get_loop(self):
         """Return the event loop used by this channel"""
 
@@ -89,6 +91,11 @@ class SSHChannel(SSHPacketHandler):
 
         return self._encoding
 
+    def set_encoding(self, encoding):
+        """Set the encoding on this channel"""
+
+        self._encoding = encoding
+
     def get_recv_window(self):
         """Return the configured receive window for this channel"""
 
@@ -103,36 +110,54 @@ class SSHChannel(SSHPacketHandler):
         """Clean up this channel"""
 
         if self._open_waiter:
-            self._open_waiter.set_exception(
-                ChannelOpenError(OPEN_CONNECT_FAILED, 'SSH connection closed'))
+            if not self._open_waiter.cancelled(): # pragma: no branch
+                self._open_waiter.set_exception(
+                    ChannelOpenError(OPEN_CONNECT_FAILED,
+                                     'SSH connection closed'))
+
             self._open_waiter = None
 
         if self._request_waiters:
             for waiter in self._request_waiters:
-                waiter.set_exception(exc)
+                if not waiter.cancelled(): # pragma: no branch
+                    waiter.set_exception(exc)
 
             self._request_waiters = []
 
-        if self._close_waiters:
-            for waiter in self._close_waiters:
-                if not waiter.cancelled():
-                    waiter.set_result(None)
-
-            self._close_waiters = []
-
         if self._session:
             self._session.connection_lost(exc)
             self._session = None
 
-        if self._conn:
-            if self._recv_chan:
-                self._conn.remove_channel(self._recv_chan)
-                self._recv_chan = None
+        self._close_event.set()
 
+        if self._conn: # pragma: no branch
+            self._conn.remove_channel(self._recv_chan)
+            self._recv_chan = None
             self._conn = None
 
-        self._send_state = 'closed'
-        self._recv_state = 'closed'
+    def _close_send(self):
+        """Discard unsent data and close the channel for sending"""
+
+        # Discard unsent data
+        self._send_buf = []
+        self._send_buf_len = 0
+
+        if self._send_state != 'closed':
+            self._send_packet(MSG_CHANNEL_CLOSE)
+            self._send_chan = None
+            self._send_state = 'closed'
+
+    def _discard_recv(self):
+        """Discard unreceived data and clean up if close received"""
+
+        # Discard unreceived data
+        self._recv_buf = []
+        self._recv_paused = False
+
+        # If recv is close_pending, we know send is already closed
+        if self._recv_state == 'close_pending':
+            self._recv_state = 'closed'
+            self._loop.call_soon(self._cleanup)
 
     def _pause_resume_writing(self):
         """Pause or resume writing based on send buffer low/high water marks"""
@@ -174,59 +199,72 @@ class SSHChannel(SSHPacketHandler):
         if not self._send_buf:
             if self._send_state == 'eof_pending':
                 self._send_packet(MSG_CHANNEL_EOF)
-                self._send_state = 'eof_sent'
+                self._send_state = 'eof'
             elif self._send_state == 'close_pending':
-                self._send_packet(MSG_CHANNEL_CLOSE)
-                self._send_state = 'close_sent'
+                self._close_send()
+
+    def _flush_recv_buf(self, exc=None):
+        """Flush as much data in the recv buffer as the application allows"""
+
+        while self._recv_buf and not self._recv_paused:
+            self._deliver_data(*self._recv_buf.pop(0))
+
+        if not self._recv_buf:
+            if self._recv_state == 'eof_pending':
+                self._recv_state = 'eof'
+
+                if (not self._session.eof_received() and
+                        self._send_state == 'open'):
+                    self.write_eof()
+            elif self._recv_state == 'close_pending':
+                self._recv_state = 'closed'
+
+                if self._recv_partial and not exc:
+                    exc = DisconnectError(DISC_PROTOCOL_ERROR,
+                                          'Unicode decode error')
+
+                self._loop.call_soon(self._cleanup, exc)
 
     def _deliver_data(self, data, datatype):
         """Deliver incoming data to the session"""
 
-        if data == _EOF:
+        self._recv_window -= len(data)
+
+        if self._recv_window < self._init_recv_window / 2:
+            self._send_packet(MSG_CHANNEL_WINDOW_ADJUST,
+                              UInt32(self._init_recv_window -
+                                     self._recv_window))
+            self._recv_window = self._init_recv_window
+
+        if self._encoding:
             if datatype in self._recv_partial:
-                raise DisconnectError(DISC_PROTOCOL_ERROR,
-                                      'Unicode decode error')
-
-            if not self._session.eof_received() and self._send_state == 'open':
-                self.write_eof()
-        else:
-            self._recv_window -= len(data)
-
-            if self._recv_window < self._init_recv_window / 2:
-                self._send_packet(MSG_CHANNEL_WINDOW_ADJUST,
-                                  UInt32(self._init_recv_window -
-                                         self._recv_window))
-                self._recv_window = self._init_recv_window
-
-            if self._encoding:
-                if datatype in self._recv_partial:
-                    encdata = self._recv_partial.pop(datatype) + data
-                else:
-                    encdata = data
-
-                while encdata:
-                    try:
-                        data = encdata.decode(self._encoding)
-                        encdata = b''
-                    except UnicodeDecodeError as exc:
-                        if exc.start > 0:
-                            # Avoid pylint false positive
-                            # pylint: disable=invalid-slice-index
-                            data = encdata[:exc.start].decode()
-                            encdata = encdata[exc.start:]
-                        elif exc.reason == 'unexpected end of data':
-                            break
-                        else:
-                            raise DisconnectError(DISC_PROTOCOL_ERROR,
-                                                  'Unicode decode error')
-
-                    self._session.data_received(data, datatype)
-
-                if encdata:
-                    self._recv_partial[datatype] = encdata
+                encdata = self._recv_partial.pop(datatype) + data
             else:
+                encdata = data
+
+            while encdata:
+                try:
+                    data = encdata.decode(self._encoding)
+                    encdata = b''
+                except UnicodeDecodeError as exc:
+                    if exc.start > 0:
+                        # Avoid pylint false positive
+                        # pylint: disable=invalid-slice-index
+                        data = encdata[:exc.start].decode()
+                        encdata = encdata[exc.start:]
+                    elif exc.reason == 'unexpected end of data':
+                        break
+                    else:
+                        raise DisconnectError(DISC_PROTOCOL_ERROR,
+                                              'Unicode decode error')
+
                 self._session.data_received(data, datatype)
 
+            if encdata:
+                self._recv_partial[datatype] = encdata
+        else:
+            self._session.data_received(data, datatype)
+
     def _accept_data(self, data, datatype=None):
         """Accept new data on the channel
 
@@ -242,10 +280,10 @@ class SSHChannel(SSHPacketHandler):
         if not data:
             return
 
-        if self._send_state in {'close_pending', 'close_sent', 'closed'}:
+        if self._send_state in {'close_pending', 'closed'}:
             return
 
-        if data != _EOF and len(data) > self._recv_window:
+        if len(data) > self._recv_window:
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Window exceeded')
 
         if self._recv_paused:
@@ -256,25 +294,29 @@ class SSHChannel(SSHPacketHandler):
     def process_connection_close(self, exc):
         """Process the SSH connection closing"""
 
-        self._cleanup(exc)
+        if self._send_state != 'closed':
+            self._send_state = 'closed'
+
+        if self._recv_state not in {'close_pending', 'closed'}:
+            self._recv_state = 'close_pending'
+            self._flush_recv_buf(exc)
+        elif self._recv_state == 'closed':
+            self._loop.call_soon(self._cleanup, exc)
 
     def process_open(self, send_chan, send_window, send_pktsize, session):
         """Process a channel open request"""
 
-        if self._recv_state != 'closed':
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Channel already open')
-
-        self._send_state = 'open_received'
         self._send_chan = send_chan
         self._send_window = send_window
         self._send_pktsize = send_pktsize
 
-        asyncio.async(self._finish_open_request(session), loop=self._loop)
+        self._conn.create_task(self._finish_open_request(session))
 
     @asyncio.coroutine
     def _finish_open_request(self, session):
         """Finish processing a channel open request"""
 
+        # pylint: disable=broad-except
         try:
             if asyncio.iscoroutine(session):
                 session = yield from session
@@ -310,8 +352,9 @@ class SSHChannel(SSHPacketHandler):
         self._send_state = 'open'
         self._recv_state = 'open'
 
-        if not self._open_waiter.cancelled():
+        if not self._open_waiter.cancelled(): # pragma: no branch
             self._open_waiter.set_result(packet)
+
         self._open_waiter = None
 
     def process_open_failure(self, code, reason, lang):
@@ -321,7 +364,10 @@ class SSHChannel(SSHPacketHandler):
             raise DisconnectError(DISC_PROTOCOL_ERROR,
                                   'Channel not being opened')
 
-        self._open_waiter.set_exception(ChannelOpenError(code, reason, lang))
+        if not self._open_waiter.cancelled(): # pragma: no branch
+            self._open_waiter.set_exception(
+                ChannelOpenError(code, reason, lang))
+
         self._open_waiter = None
         self._loop.call_soon(self._cleanup)
 
@@ -330,7 +376,7 @@ class SSHChannel(SSHPacketHandler):
 
         # pylint: disable=unused-argument
 
-        if self._recv_state not in {'open', 'eof_received'}:
+        if self._recv_state not in {'open', 'eof_pending', 'eof'}:
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Channel not open')
 
         adjust = packet.get_uint32()
@@ -383,41 +429,32 @@ class SSHChannel(SSHPacketHandler):
 
         packet.check_end()
 
-        self._recv_state = 'eof_received'
-        self._accept_data(_EOF)
+        self._recv_state = 'eof_pending'
+        self._flush_recv_buf()
 
     def _process_close(self, pkttype, packet):
         """Process an incoming channel close"""
 
         # pylint: disable=unused-argument
 
-        if self._recv_state not in {'open', 'eof_received'}:
+        if self._recv_state not in {'open', 'eof_pending', 'eof'}:
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Channel not open')
 
         packet.check_end()
 
-        # Flush any unsent data
-        self._send_buf = []
-        self._send_buf_len = 0
+        self._close_send()
 
-        # If we haven't yet sent a close, send one now
-        if self._send_state not in {'close_sent', 'closed'}:
-            self._send_packet(MSG_CHANNEL_CLOSE)
-            self._send_state = 'close_sent'
-
-        self._loop.call_soon(self._cleanup)
+        self._recv_state = 'close_pending'
+        self._flush_recv_buf()
 
     def _process_request(self, pkttype, packet):
         """Process an incoming channel request"""
 
         # pylint: disable=unused-argument
 
-        if self._recv_state not in {'open', 'eof_received'}:
+        if self._recv_state not in {'open', 'eof_pending', 'eof'}:
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Channel not open')
 
-        if self._send_state in {'close_pending', 'close_sent', 'closed'}:
-            return
-
         request = packet.get_string()
         want_reply = packet.get_boolean()
 
@@ -427,34 +464,28 @@ class SSHChannel(SSHPacketHandler):
             raise DisconnectError(DISC_PROTOCOL_ERROR,
                                   'Invalid channel request') from None
 
-        name = '_process_' + request.replace('-', '_') + '_request'
+        name = '_process_' + map_handler_name(request) + '_request'
         handler = getattr(self, name, None)
         result = handler(packet) if callable(handler) else False
 
-        if want_reply:
+        if want_reply and self._send_state not in {'close_pending', 'closed'}:
             if result:
                 self._send_packet(MSG_CHANNEL_SUCCESS)
             else:
                 self._send_packet(MSG_CHANNEL_FAILURE)
 
-        if result and request in ('shell', 'exec', 'subsystem'):
+        if result and request in {'shell', 'exec', 'subsystem'}:
             self._session.session_started()
             self.resume_reading()
 
     def _process_response(self, pkttype, packet):
         """Process a success or failure response"""
 
-        # pylint: disable=unused-argument
-
-        if self._send_state not in {'open', 'eof_pending', 'eof_sent',
-                                    'close_pending', 'close_sent'}:
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Channel not open')
-
         packet.check_end()
 
         if self._request_waiters:
             waiter = self._request_waiters.pop(0)
-            if not waiter.cancelled():
+            if not waiter.cancelled(): # pragma: no branch
                 waiter.set_result(pkttype == MSG_CHANNEL_SUCCESS)
         else:
             raise DisconnectError(DISC_PROTOCOL_ERROR,
@@ -485,14 +516,13 @@ class SSHChannel(SSHPacketHandler):
                                UInt32(self._recv_window),
                                UInt32(self._recv_pktsize), *args)
 
-        self._send_state = 'open_sent'
         return (yield from self._open_waiter)
 
     def _send_packet(self, pkttype, *args):
         """Send a packet on the channel"""
 
-        if self._send_chan is None:
-            raise OSError('Channel not open')
+        if self._send_chan is None: # pragma: no cover
+            return
 
         self._conn.send_packet(Byte(pkttype), UInt32(self._send_chan), *args)
 
@@ -506,6 +536,9 @@ class SSHChannel(SSHPacketHandler):
     def _make_request(self, request, *args):
         """Make a channel request and wait for the response"""
 
+        if self._send_chan is None:
+            return False
+
         waiter = asyncio.Future(loop=self._loop)
         self._request_waiters.append(waiter)
         self._send_request(request, *args, want_reply=True)
@@ -520,9 +553,13 @@ class SSHChannel(SSHPacketHandler):
 
         """
 
-        if self._send_state not in {'close_sent', 'closed'}:
-            self._send_packet(MSG_CHANNEL_CLOSE)
-            self._send_state = 'close_sent'
+        if self._send_state not in {'close_pending', 'closed'}:
+            # Send an immediate close, discarding unsent data
+            self._close_send()
+
+        if self._recv_state != 'closed':
+            # Discard unreceived data
+            self._discard_recv()
 
     def close(self):
         """Cleanly close the channel
@@ -534,10 +571,15 @@ class SSHChannel(SSHPacketHandler):
 
         """
 
-        if self._send_state not in {'close_pending', 'close_sent', 'closed'}:
+        if self._send_state not in {'close_pending', 'closed'}:
+            # Send a close only after sending unsent data
             self._send_state = 'close_pending'
             self._flush_send_buf()
 
+        if self._recv_state != 'closed':
+            # Discard unreceived data
+            self._discard_recv()
+
     @asyncio.coroutine
     def wait_closed(self):
         """Wait for this channel to close
@@ -547,10 +589,7 @@ class SSHChannel(SSHPacketHandler):
 
         """
 
-        if self._session:
-            waiter = asyncio.Future(loop=self._loop)
-            self._close_waiters.append(waiter)
-            yield from waiter
+        yield from self._close_event.wait()
 
     def get_extra_info(self, name, default=None):
         """Get additional information about the channel
@@ -564,6 +603,12 @@ class SSHChannel(SSHPacketHandler):
            ``'remote_peername'`` are added to return the local and remote
            host and port information for the tunneled TCP connection.
 
+           For UNIX channels, the values ``'local_peername'`` and
+           ``'remote_peername'`` are added to return the local and remote
+           path information for the tunneled UNIX domain socket connection.
+           Since UNIX domain sockets provide no "source" address, only
+           one of these will be filled in.
+
            See :meth:`get_extra_info() <SSHClientConnection.get_extra_info>`
            on :class:`SSHClientConnection` for more information.
 
@@ -631,10 +676,10 @@ class SSHChannel(SSHPacketHandler):
 
            :param data:
                The data to send on the channel
-           :param integer datatype: (optional)
+           :param int datatype: (optional)
                The extended data type of the data, from :ref:`extended
                data types <ExtendedDataTypes>`
-           :type data: string or bytes
+           :type data: str or bytes
 
            :raises: :exc:`OSError` if the channel isn't open for sending
                     or the extended data type is not valid for this type
@@ -667,10 +712,10 @@ class SSHChannel(SSHPacketHandler):
 
            :param list_of_data:
                The data to send on the channel
-           :param integer datatype: (optional)
+           :param int datatype: (optional)
                The extended data type of the data, from :ref:`extended
                data types <ExtendedDataTypes>`
-           :type list_of_data: iterable of ``string`` or ``bytes`` objects
+           :type list_of_data: iterable of str or bytes objects
 
            :raises: :exc:`OSError` if the channel isn't open for sending
                     or the extended data type is not valid for this type
@@ -693,11 +738,9 @@ class SSHChannel(SSHPacketHandler):
 
         """
 
-        if self._send_state != 'open':
-            raise BrokenPipeError('Channel not open for sending')
-
-        self._send_state = 'eof_pending'
-        self._flush_send_buf()
+        if self._send_state == 'open':
+            self._send_state = 'eof_pending'
+            self._flush_send_buf()
 
     def pause_reading(self):
         """Pause delivery of incoming data
@@ -724,16 +767,14 @@ class SSHChannel(SSHPacketHandler):
            This method can be called to resume delivery of incoming data
            which was suspended by a call to :meth:`pause_reading`. As soon
            as this method is called, any buffered data will be delivered
-           immediately.  A pending end-of-file notication may also be
+           immediately. A pending end-of-file notication may also be
            delivered if one was queued while reading was paused.
 
         """
 
         if self._recv_paused:
             self._recv_paused = False
-
-            while self._recv_buf and not self._recv_paused:
-                self._deliver_data(*self._recv_buf.pop(0))
+            self._flush_recv_buf()
 
 
 class SSHClientChannel(SSHChannel):
@@ -749,7 +790,7 @@ class SSHClientChannel(SSHChannel):
 
     @asyncio.coroutine
     def create(self, session_factory, command, subsystem, env,
-               term_type, term_size, term_modes):
+               term_type, term_size, term_modes, agent_forwarding):
         """Create an SSH client session"""
 
         packet = yield from self._open(b'session')
@@ -761,20 +802,16 @@ class SSHClientChannel(SSHChannel):
         self._session.connection_made(self)
 
         for name, value in env.items():
-            name = str(name).encode('utf-8')
-            value = str(value).encode('utf-8')
-            self._send_request(b'env', String(name), String(value))
+            self._send_request(b'env', String(str(name)), String(str(value)))
 
         if term_type:
-            term_type = term_type.encode('ascii')
-
-            if len(term_size) == 4:
-                width, height, pixwidth, pixheight = term_size
+            if not term_size:
+                width = height = pixwidth = pixheight = 0
             elif len(term_size) == 2:
                 width, height = term_size
                 pixwidth = pixheight = 0
-            elif not term_size:
-                width = height = pixwidth = pixheight = 0
+            elif len(term_size) == 4:
+                width, height, pixwidth, pixheight = term_size
             else:
                 raise ValueError('If set, terminal size must be a tuple of '
                                  '2 or 4 integers')
@@ -799,6 +836,9 @@ class SSHClientChannel(SSHChannel):
                 raise ChannelOpenError(OPEN_REQUEST_PTY_FAILED,
                                        'PTY request failed')
 
+        if agent_forwarding:
+            self._send_request(b'auth-agent-req@openssh.com')
+
         if command:
             result = yield from self._make_request(b'exec', String(command))
         elif subsystem:
@@ -812,10 +852,6 @@ class SSHClientChannel(SSHChannel):
             raise ChannelOpenError(OPEN_REQUEST_SESSION_FAILED,
                                    'Session request failed')
 
-        if not self._session:
-            raise ChannelOpenError(OPEN_REQUEST_SESSION_FAILED,
-                                   'Channel closed during session startup')
-
         self._session.session_started()
         self.resume_reading()
 
@@ -899,13 +935,13 @@ class SSHClientChannel(SSHChannel):
            This method changes the width and height of the terminal
            associated with this session.
 
-           :param integer width:
+           :param int width:
                The width of the terminal in characters
-           :param integer height:
+           :param int height:
                The height of the terminal in characters
-           :param integer pixwidth: (optional)
+           :param int pixwidth: (optional)
                The width of the terminal in pixels
-           :param integer pixheight: (optional)
+           :param int pixheight: (optional)
                The height of the terminal in pixels
 
         """
@@ -920,7 +956,7 @@ class SSHClientChannel(SSHChannel):
            operation on the remote process or service as described in
            :rfc:`4335`.
 
-           :param integer msec:
+           :param int msec:
                The duration of the break in milliseconds
 
            :raises: :exc:`OSError` if the channel is not open
@@ -936,15 +972,13 @@ class SSHClientChannel(SSHChannel):
            process or service. Signal names should be as described in
            section 6.10 of :rfc:`4254#section-6.10`.
 
-           :param string signal:
+           :param str signal:
                The signal to deliver
 
            :raises: :exc:`OSError` if the channel is not open
 
         """
 
-        signal = signal.encode('ascii')
-
         self._send_request(b'signal', String(signal))
 
     def terminate(self):
@@ -962,7 +996,7 @@ class SSHClientChannel(SSHChannel):
     def kill(self):
         """Forcibly kill the remote process
 
-           This method can be called to forcibly stop  the remote process
+           This method can be called to forcibly stop the remote process
            or service by sending it a ``KILL`` signal.
 
            :raises: :exc:`OSError` if the channel is not open
@@ -977,11 +1011,14 @@ class SSHServerChannel(SSHChannel):
 
     _write_datatypes = {EXTENDED_DATA_STDERR}
 
-    def __init__(self, conn, loop, encoding, window, max_pktsize):
+    def __init__(self, conn, loop, allow_pty, agent_forwarding,
+                 encoding, window, max_pktsize):
         """Initialize an SSH server channel"""
 
         super().__init__(conn, loop, encoding, window, max_pktsize)
 
+        self._allow_pty = allow_pty
+        self._agent_forwarding = agent_forwarding
         self._env = self._conn.get_key_option('environment', {})
         self._command = None
         self._subsystem = None
@@ -1006,7 +1043,8 @@ class SSHServerChannel(SSHChannel):
             raise DisconnectError(DISC_PROTOCOL_ERROR,
                                   'Invalid pty request') from None
 
-        if not self._conn.check_key_permission('pty') or \
+        if not self._allow_pty or \
+           not self._conn.check_key_permission('pty') or \
            not self._conn.check_certificate_permission('pty'):
             return False
 
@@ -1030,6 +1068,19 @@ class SSHServerChannel(SSHChannel):
         return self._session.pty_requested(self._term_type, self._term_size,
                                            self._term_modes)
 
+    def _process_auth_agent_req_at_openssh_dot_com_request(self, packet):
+        """Process a request to enable ssh-agent forwarding"""
+
+        packet.check_end()
+
+        if not self._agent_forwarding or \
+           not self._conn.check_key_permission('agent-forwarding') or \
+           not self._conn.check_certificate_permission('agent-forwarding'):
+            return False
+
+        self._conn.agent_forwarding_enabled()
+        return True
+
     def _process_env_request(self, packet):
         """Process a request to set an environment variable"""
 
@@ -1136,36 +1187,6 @@ class SSHServerChannel(SSHChannel):
 
         return self._session.break_received(msec)
 
-    def start_sftp_server(self, sftp_factory):
-        """Start an SFTP server for this session
-
-           This method can be used by an existing :class:`SSHServerSession`
-           to replace itself with an SFTP server session. Calls to this
-           method should be made from  :meth:`session_started
-           <SSHServerSession.session_started>` before any data is read
-           or written.  Once called, no further calls will be made on the
-           original session.
-
-             .. note:: The :meth:`connection_lost
-                       <SSHServerSession.connection_lost>` method will not
-                       be called on the original server session when this
-                       is used.
-
-           :param callable sftp_server:
-               A callable which returns an :class:`SFTPServer` object
-               that will be created to handle SFTP requests on this
-               channel.
-
-        """
-
-        # Reset the encoding to allow the transfer of binary data
-        self._encoding = None
-
-        # Replace the session with an SFTPServerSession
-        self._session = SFTPServerSession(sftp_factory(self._conn))
-        self._session.connection_made(self)
-        self._session.session_started()
-
     def get_environment(self):
         """Return the environment for this session
 
@@ -1226,7 +1247,7 @@ class SSHServerChannel(SSHChannel):
            calls to this can be made at any time after the handler
            function has started up.
 
-           :returns: A string containing the terminal type or ``None`` if
+           :returns: A str containing the terminal type or ``None`` if
                      no pseudo-terminal was requested
 
         """
@@ -1271,11 +1292,11 @@ class SSHServerChannel(SSHChannel):
            calls to this can be made at any time after the handler
            function has started up.
 
-           :param integer mode:
+           :param int mode:
                POSIX terminal mode taken from :ref:`POSIX terminal modes
                <PTYModes>` to look up
 
-           :returns: An integer containing the value of the requested
+           :returns: An int containing the value of the requested
                      POSIX terminal mode or ``None`` if the requested
                      mode was not set
 
@@ -1294,7 +1315,7 @@ class SSHServerChannel(SSHChannel):
            enable this functionality or to ``False`` to tell the client
            to forward Control-S and Control-Q through as normal input.
 
-           :param boolean client_can_do:
+           :param bool client_can_do:
                Whether or not the client should enable XON/XOFF flow control
 
         """
@@ -1312,7 +1333,7 @@ class SSHServerChannel(SSHChannel):
 
            :param data:
                The data to send to stderr
-           :type data: string or bytes
+           :type data: str or bytes
 
            :raises: :exc:`OSError` if the channel isn't open for sending
 
@@ -1340,18 +1361,16 @@ class SSHServerChannel(SSHChannel):
            successful. After reporting the status, the channel is
            closed.
 
-           :param integer status:
+           :param int status:
                The exit status to report to the client
 
            :raises: :exc:`OSError` if the channel isn't open
 
         """
 
-        if self._send_state not in {'open', 'eof_pending', 'eof_sent'}:
-            raise OSError('Channel not open')
-
-        self._send_request(b'exit-status', UInt32(status & 0xff))
-        self.close()
+        if self._send_state not in {'close_pending', 'closed'}:
+            self._send_request(b'exit-status', UInt32(status & 0xff))
+            self.close()
 
     def exit_with_signal(self, signal, core_dumped=False,
                          msg='', lang=DEFAULT_LANG):
@@ -1363,37 +1382,31 @@ class SSHServerChannel(SSHChannel):
            of whether or not the process dumped core. After
            reporting the signal, the channel is closed.
 
-           :param string signal:
+           :param str signal:
                The signal which caused the process to exit
-           :param boolean core_dumped: (optional)
+           :param bool core_dumped: (optional)
                Whether or not the process dumped core
-           :param msg: (optional)
+           :param str msg: (optional)
                Details about what error occurred
-           :param lang: (optional)
+           :param str lang: (optional)
                The language the error message is in
 
            :raises: :exc:`OSError` if the channel isn't open
 
         """
 
-        if self._send_state not in {'open', 'eof_pending', 'eof_sent'}:
-            raise OSError('Channel not open')
-
-        signal = signal.encode('ascii')
-        msg = msg.encode('utf-8')
-        lang = lang.encode('ascii')
-
-        self._send_request(b'exit-signal', String(signal),
-                           Boolean(core_dumped), String(msg), String(lang))
-        self.close()
+        if self._send_state not in {'close_pending', 'closed'}:
+            self._send_request(b'exit-signal', String(signal),
+                               Boolean(core_dumped), String(msg), String(lang))
+            self.close()
 
 
-class SSHTCPChannel(SSHChannel):
-    """SSH TCP channel"""
+class SSHForwardChannel(SSHChannel):
+    """SSH channel for forwarding TCP and UNIX domain connections"""
 
     @asyncio.coroutine
     def _finish_open_request(self, session):
-        """Finish processing a TCP channel open request"""
+        """Finish processing a forward channel open request"""
 
         yield from super()._finish_open_request(session)
 
@@ -1402,28 +1415,38 @@ class SSHTCPChannel(SSHChannel):
             self.resume_reading()
 
     @asyncio.coroutine
-    def _open_tcp(self, session_factory, chantype, host, port,
-                  orig_host, orig_port):
-        """Open a TCP channel"""
+    def _open(self, session_factory, chantype, *args):
+        """Open a forward channel"""
 
-        self._extra['local_peername'] = (orig_host, orig_port)
-        self._extra['remote_peername'] = (host, port)
+        packet = yield from super()._open(chantype, *args)
 
-        host = host.encode('utf-8')
-        orig_host = orig_host.encode('utf-8')
-
-        packet = yield from super()._open(chantype, String(host), UInt32(port),
-                                          String(orig_host), UInt32(orig_port))
-
-        # TCP sessions should have no extra data in the open confirmation
+        # Forward channels should have no extra data in the open confirmation
         packet.check_end()
 
         self._session = session_factory()
         self._session.connection_made(self)
+
         self._session.session_started()
         self.resume_reading()
 
-        return self, self._session
+        return self._session
+
+
+class SSHTCPChannel(SSHForwardChannel):
+    """SSH TCP channel"""
+
+    @asyncio.coroutine
+    def _open_tcp(self, session_factory, chantype, host, port,
+                  orig_host, orig_port):
+        """Open a TCP channel"""
+
+        self._extra['peername'] = None
+        self._extra['local_peername'] = (orig_host, orig_port)
+        self._extra['remote_peername'] = (host, port)
+
+        return (yield from super()._open(session_factory, chantype,
+                                         String(host), UInt32(port),
+                                         String(orig_host), UInt32(orig_port)))
 
     @asyncio.coroutine
     def connect(self, session_factory, host, port, orig_host, orig_port):
@@ -1445,3 +1468,52 @@ class SSHTCPChannel(SSHChannel):
 
         self._extra['local_peername'] = (dest_host, dest_port)
         self._extra['remote_peername'] = (orig_host, orig_port)
+
+
+class SSHUNIXChannel(SSHForwardChannel):
+    """SSH UNIX channel"""
+
+    @asyncio.coroutine
+    def _open_unix(self, session_factory, chantype, path, *args):
+        """Open a UNIX channel"""
+
+        self._extra['local_peername'] = ''
+        self._extra['remote_peername'] = path
+
+        return (yield from super()._open(session_factory, chantype,
+                                         String(path), *args))
+
+    @asyncio.coroutine
+    def connect(self, session_factory, path):
+        """Create a new outbound UNIX session"""
+
+        # OpenSSH appears to have a bug which requires an originator
+        # host and port to be sent after the path name to connect to
+        # when opening a direct streamlocal channel.
+        return (yield from self._open_unix(session_factory,
+                                           b'direct-streamlocal@openssh.com',
+                                           path, String(''), UInt32(0)))
+
+    @asyncio.coroutine
+    def accept(self, session_factory, path):
+        """Create a new forwarded UNIX session"""
+
+        return (yield from self._open_unix(session_factory,
+                                           b'forwarded-streamlocal@openssh.com',
+                                           path, String('')))
+
+    def set_inbound_peer_names(self, dest_path):
+        """Set local and remote peer names for inbound connections"""
+
+        self._extra['local_peername'] = dest_path
+        self._extra['remote_peername'] = ''
+
+class SSHAgentChannel(SSHForwardChannel):
+    """SSH agent channel"""
+
+    @asyncio.coroutine
+    def open(self, session_factory):
+        """Open an SSH agent channel"""
+
+        return (yield from self._open(session_factory,
+                                      b'auth-agent@openssh.com'))
