@@ -16,7 +16,7 @@ from .asn1 import ASN1DecodeError, ObjectIdentifier, der_encode, der_decode
 from .crypto import DSAPrivateKey, DSAPublicKey
 from .misc import all_ints
 from .packet import MPInt, String, PacketDecodeError, SSHPacket
-from .public_key import SSHKey, SSHCertificateV00, SSHCertificateV01
+from .public_key import SSHKey, SSHCertificateV01
 from .public_key import KeyExportError
 from .public_key import register_public_key_alg, register_certificate_alg
 
@@ -38,7 +38,7 @@ class _DSAKey(SSHKey):
         # This isn't protected access - both objects are _DSAKey instances
         # pylint: disable=protected-access
 
-        return (isinstance(other, self.__class__) and
+        return (isinstance(other, type(self)) and
                 self._key.p == other._key.p and
                 self._key.q == other._key.q and
                 self._key.g == other._key.g and
@@ -219,5 +219,3 @@ register_public_key_alg(b'ssh-dss', _DSAKey)
 
 register_certificate_alg(b'ssh-dss-cert-v01@openssh.com',
                          _DSAKey, SSHCertificateV01)
-register_certificate_alg(b'ssh-dss-cert-v00@openssh.com',
-                         _DSAKey, SSHCertificateV00)
