@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.4
+#!/usr/bin/env python3.5
 #
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -14,14 +14,13 @@
 
 import asyncio, asyncssh, sys
 
-@asyncio.coroutine
-def run_client():
-    with (yield from asyncssh.connect('localhost')) as conn:
-        stdin, stdout, stderr = yield from conn.open_session('bc')
+async def run_client():
+    async with asyncssh.connect('localhost') as conn:
+        stdin, stdout, stderr = await conn.open_session('bc')
 
         for op in ['2+2', '1*2*3*4', '2^32']:
             stdin.write(op + '\n')
-            result = yield from stdout.readline()
+            result = await stdout.readline()
             print(op, '=', result, end='')
 
 try:
