@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2014-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -18,10 +18,10 @@
 """
 
 import binascii
-import importlib.util
 import os
 
-from .util import TempDirTestCase, run
+from .util import bcrypt_available, libnacl_available
+from .util import make_certificate, run, TempDirTestCase
 
 from asyncssh import import_private_key, import_public_key, import_certificate
 from asyncssh import read_private_key, read_public_key, read_certificate
@@ -30,15 +30,12 @@ from asyncssh import read_certificate_list
 from asyncssh import KeyImportError, KeyExportError, KeyEncryptionError
 from asyncssh.asn1 import der_encode, BitString, ObjectIdentifier
 from asyncssh.asn1 import TaggedDERObject
-from asyncssh.packet import MPInt, String, UInt32, UInt64
+from asyncssh.packet import MPInt, String, UInt32
 from asyncssh.pbe import pkcs1_decrypt
 from asyncssh.public_key import CERT_TYPE_USER, CERT_TYPE_HOST, SSHKey
 from asyncssh.public_key import get_public_key_algs, get_certificate_algs
 
 
-bcrypt_available = importlib.util.find_spec('bcrypt')
-libnacl_available = importlib.util.find_spec('libnacl')
-
 _ES1_SHA1_DES = ObjectIdentifier('1.2.840.113549.1.5.10')
 _P12_RC4_40 = ObjectIdentifier('1.2.840.113549.1.12.1.2')
 _ES2 = ObjectIdentifier('1.2.840.113549.1.5.13')
@@ -62,14 +59,6 @@ pkcs8_ciphers = (('des-cbc',      'md5',    1, '-v1 PBE-MD5-DES'),
                  ('rc4-40',       'sha1',   1, '-v1 PBE-SHA1-RC4-40'),
                  ('rc4-128',      'sha1',   1, '-v1 PBE-SHA1-RC4-128'),
                  ('aes128-cbc',   'sha1',   2, '-v2 aes-128-cbc'),
-                 ('aes128-cbc',   'sha224', 2, '-v2 aes-128-cbc '
-                                               '-v2prf hmacWithSHA224'),
-                 ('aes128-cbc',   'sha256', 2, '-v2 aes-128-cbc '
-                                               '-v2prf hmacWithSHA256'),
-                 ('aes128-cbc',   'sha384', 2, '-v2 aes-128-cbc '
-                                               '-v2prf hmacWithSHA384'),
-                 ('aes128-cbc',   'sha512', 2, '-v2 aes-128-cbc '
-                                               '-v2prf hmacWithSHA512'),
                  ('aes192-cbc',   'sha1',   2, '-v2 aes-192-cbc'),
                  ('aes256-cbc',   'sha1',   2, '-v2 aes-256-cbc'),
                  ('blowfish-cbc', 'sha1',   2, '-v2 bf-cbc'),
@@ -82,6 +71,36 @@ openssh_ciphers = ('aes128-cbc', 'aes192-cbc', 'aes256-cbc',
                    'arcfour', 'arcfour128', 'arcfour256',
                    'blowfish-cbc', 'cast128-cbc', '3des-cbc')
 
+# pylint: enable=bad-whitespace
+
+_openssl_version = run('openssl version')
+
+_pkcs1_public_supported = _openssl_version >= b'OpenSSL 1.0.0'
+
+if _openssl_version >= b'OpenSSL 1.0.2': # pragma: no branch
+    # pylint: disable=bad-whitespace
+
+    pkcs8_ciphers += (
+        ('aes128-cbc',   'sha224', 2, '-v2 aes-128-cbc '
+                                      '-v2prf hmacWithSHA224'),
+        ('aes128-cbc',   'sha256', 2, '-v2 aes-128-cbc '
+                                      '-v2prf hmacWithSHA256'),
+        ('aes128-cbc',   'sha384', 2, '-v2 aes-128-cbc '
+                                      '-v2prf hmacWithSHA384'),
+        ('aes128-cbc',   'sha512', 2, '-v2 aes-128-cbc '
+                                      '-v2prf hmacWithSHA512')
+    )
+
+if run('ssh -V') >= b'OpenSSH_6.9': # pragma: no branch
+    # GCM & Chacha tests require OpenSSH 6.9 due to a bug in earlier versions:
+    #     https://bugzilla.mindrot.org/show_bug.cgi?id=2366
+
+    openssh_ciphers += ('aes128-gcm@openssh.com', 'aes256-gcm@openssh.com')
+
+    # Only test Chacha if libnacl is installed
+    if libnacl_available: # pragma: no branch
+        openssh_ciphers += ('chacha20-poly1305@openssh.com',)
+
 
 def select_passphrase(cipher, pbe_version=0):
     """Randomize between string and bytes version of passphrase"""
@@ -96,15 +115,6 @@ def select_passphrase(cipher, pbe_version=0):
     else:
         return 'passphrase'.encode('utf-8')
 
-# pylint: enable=bad-whitespace
-
-if run('ssh -V') >= b'OpenSSH_6.9': # pragma: no branch
-    # GCM & Chacha tests require OpenSSH 6.9 due to a bug in earlier versions:
-    #     https://bugzilla.mindrot.org/show_bug.cgi?id=2366
-    openssh_ciphers = openssh_ciphers + ('aes128-gcm@openssh.com',
-                                         'aes256-gcm@openssh.com',
-                                         'chacha20-poly1305@openssh.com')
-
 
 class _TestPublicKey(TempDirTestCase):
     """Unit tests for public key modules"""
@@ -115,7 +125,6 @@ class _TestPublicKey(TempDirTestCase):
     private_formats = ()
     public_formats = ()
     default_cert_version = ''
-    cert_versions = ()
 
     def __init__(self, methodName='runTest'):
         super().__init__(methodName)
@@ -130,39 +139,10 @@ class _TestPublicKey(TempDirTestCase):
 
         raise NotImplementedError
 
-    def encode_options(self, options):
-        """Encode SSH certificate critical options and extensions"""
-
-        # pylint: disable=no-self-use
-
-        return b''.join((String(k) + String(v) for k, v in options.items()))
-
-    def make_certificate(self, cert_type, key, signing_key, principals,
-                         valid_after=0, valid_before=0xffffffffffffffff,
-                         options=None, extensions=None, bad_signature=False):
+    def make_certificate(self, *args, **kwargs):
         """Construct an SSH certificate"""
 
-        keydata = key.encode_ssh_public()
-        principals = b''.join((String(p) for p in principals))
-        options = self.encode_options(options) if options else b''
-        extensions = self.encode_options(extensions) if extensions else b''
-        signing_keydata = b''.join((String(signing_key.algorithm),
-                                    signing_key.encode_ssh_public()))
-
-        data = b''.join((String(self.default_cert_version),
-                         String(os.urandom(8)), keydata, UInt64(0),
-                         UInt32(cert_type), String(''), String(principals),
-                         UInt64(valid_after), UInt64(valid_before),
-                         String(options), String(extensions), String(''),
-                         String(signing_keydata)))
-
-        if bad_signature:
-            data += String('')
-        else:
-            data += String(signing_key.sign(data))
-
-        return b''.join((self.default_cert_version.encode('ascii'), b' ',
-                         binascii.b2a_base64(data)))
+        return make_certificate(self.default_cert_version, *args, **kwargs)
 
     def check_private(self, passphrase=None):
         """Check for a private key match"""
@@ -396,7 +376,8 @@ class _TestPublicKey(TempDirTestCase):
                     self.privkey.export_private_key('pkcs8-pem', 'x',
                                                     'aes128-cbc', 'sha1', 3)
 
-        if 'openssh' in self.private_formats: # pragma: no branch
+        if ('openssh' in self.private_formats and # pragma: no branch
+                bcrypt_available):
             with self.subTest('Encode with unknown openssh cipher'):
                 with self.assertRaises(KeyEncryptionError):
                     self.privkey.export_private_key('openssh', 'x', 'xxx')
@@ -871,22 +852,18 @@ class _TestPublicKey(TempDirTestCase):
         with self.subTest('Export RFC4716 public'):
             self.export_rfc4716_public()
 
-    def check_certificate(self, cert_type, version, fmt):
+    def check_certificate(self, cert_type, fmt):
         """Check SSH certificate import"""
 
         with self.subTest('Import certificate'):
             typearg = '-h ' if cert_type == CERT_TYPE_HOST else ''
 
-            if version:
-                version = '-t ' + version + ' '
-
             if cert_type == CERT_TYPE_USER:
                 options = '-O force-command=xxx -O source-address=127.0.0.1 '
             else:
                 options = ''
 
-            run('ssh-keygen -s privca %s%s%s-I name sshpub' %
-                (typearg, version, options))
+            run('ssh-keygen -s privca %s%s-I name sshpub' % (typearg, options))
 
             if fmt == 'openssh':
                 run('mv sshpub-cert.pub cert')
@@ -1041,7 +1018,7 @@ class _TestPublicKey(TempDirTestCase):
                 if 'pkcs1' in self.private_formats:
                     self.check_pkcs1_private()
 
-                if 'pkcs1' in self.public_formats:
+                if 'pkcs1' in self.public_formats and _pkcs1_public_supported:
                     self.check_pkcs1_public()
 
                 if 'pkcs8' in self.private_formats:
@@ -1060,11 +1037,9 @@ class _TestPublicKey(TempDirTestCase):
                     self.check_rfc4716_public()
 
                 for cert_type in (CERT_TYPE_USER, CERT_TYPE_HOST):
-                    for version in self.cert_versions:
-                        for fmt in ('openssh', 'rfc4716'):
-                            with self.subTest(cert_type=cert_type,
-                                              version=version, fmt=fmt):
-                                self.check_certificate(cert_type, version, fmt)
+                    for fmt in ('openssh', 'rfc4716'):
+                        with self.subTest(cert_type=cert_type, fmt=fmt):
+                            self.check_certificate(cert_type, fmt)
 
                     self.check_certificate_errors(cert_type)
 
@@ -1077,7 +1052,6 @@ class TestDSA(_TestPublicKey):
     private_formats = ('pkcs1', 'pkcs8', 'openssh')
     public_formats = ('pkcs1', 'pkcs8', 'openssh', 'rfc4716')
     default_cert_version = 'ssh-dss-cert-v01@openssh.com'
-    cert_versions = ('ssh-dss-cert-v00@openssh.com', '')
 
     def make_keypair(self, privfile, pubfile, keytype):
         """Make a DSA key pair"""
@@ -1096,7 +1070,6 @@ class TestRSA(_TestPublicKey):
     private_formats = ('pkcs1', 'pkcs8', 'openssh')
     public_formats = ('pkcs1', 'pkcs8', 'openssh', 'rfc4716')
     default_cert_version = 'ssh-rsa-cert-v01@openssh.com'
-    cert_versions = ('ssh-rsa-cert-v00@openssh.com', '')
 
     def make_keypair(self, privfile, pubfile, keytype):
         """Make an RSA key pair"""
@@ -1114,7 +1087,6 @@ class TestEC(_TestPublicKey):
     base_format = 'pkcs8'
     private_formats = ('pkcs1', 'pkcs8', 'openssh')
     public_formats = ('pkcs8', 'openssh', 'rfc4716')
-    cert_versions = ('',)
 
     @property
     def default_cert_version(self):
@@ -1141,7 +1113,6 @@ if libnacl_available: # pragma: no branch
         private_formats = ('openssh')
         public_formats = ('openssh', 'rfc4716')
         default_cert_version = 'ssh-ed25519-cert-v01@openssh.com'
-        cert_versions = ('',)
 
         def make_keypair(self, privfile, pubfile, keytype):
             """Make an Ed25519 key pair"""
@@ -1171,10 +1142,12 @@ class _TestPublicKeyTopLevel(TempDirTestCase):
     def test_ec_explicit(self):
         """Test EC certificate with explcit parameters"""
 
-        with self.subTest('Import EC key with explicit parameters'):
-            run('openssl ecparam -out priv -noout -genkey -name secp256r1 '
-                '-param_enc explicit')
-            read_private_key('priv')
+        for curve in ('secp256r1', 'secp384r1', 'secp521r1'):
+            with self.subTest('Import EC key with explicit parameters',
+                              curve=curve):
+                run('openssl ecparam -out priv -noout -genkey -name %s '
+                    '-param_enc explicit' % curve)
+                read_private_key('priv')
 
         with self.subTest('Import EC key with unknown explicit parameters'):
             run('openssl ecparam -out priv -noout -genkey -name secp112r1 '
